@@ -1,15 +1,16 @@
+import json
+import logging
 import os
+import pathlib
 import shlex
 import time
-import pathlib
-import logging
-from .exceptions import *
-from .general import *
-from .disk import Partition
+from .disk import Partition, convert_device_to_uuid
+from .general import SysCommand, SysCommandWorker
 from .output import log
+from .exceptions import SysCallError, DiskError
 from .storage import storage
 
-class luks2():
+class luks2:
 	def __init__(self, partition, mountpoint, password, key_file=None, auto_unmount=False, *args, **kwargs):
 		self.password = password
 		self.partition = partition
@@ -22,12 +23,9 @@ class luks2():
 		self.mapdev = None
 
 	def __enter__(self):
-		#if self.partition.allow_formatting:
-		#	self.key_file = self.encrypt(self.partition, *self.args, **self.kwargs)
-		#else:
 		if not self.key_file:
 			self.key_file = f"/tmp/{os.path.basename(self.partition.path)}.disk_pw"  # TODO: Make disk-pw-file randomly unique?
-		
+
 		if type(self.password) != bytes:
 			self.password = bytes(self.password, 'UTF-8')
 
@@ -46,9 +44,6 @@ class luks2():
 		return True
 
 	def encrypt(self, partition, password=None, key_size=512, hash_type='sha512', iter_time=10000, key_file=None):
-		if not self.partition.allow_formatting:
-			raise DiskError(f'Could not encrypt volume {self.partition} due to it having a formatting lock.')
-
 		log(f'Encrypting {partition} (This might take a while)', level=logging.INFO)
 
 		if not key_file:
@@ -66,6 +61,8 @@ class luks2():
 		with open(key_file, 'wb') as fh:
 			fh.write(password)
 
+		SysCommand(f'bash -c "partprobe"') # Might be redundant
+
 		cryptsetup_args = shlex.join([
 			'/usr/bin/cryptsetup',
 			'--batch-mode',
@@ -81,8 +78,17 @@ class luks2():
 		])
 
 		try:
-			# Try to setup the crypt-device
-			cmd_handle = sys_command(cryptsetup_args)
+			# Retry formatting the volume because archinstall can some times be too quick
+			# which generates a "Device /dev/sdX does not exist or access denied." between
+			# setting up partitions and us trying to encrypt it.
+			for i in range(storage['DISK_RETRY_ATTEMPTS']):
+				if (cmd_handle := SysCommand(cryptsetup_args)).exit_code != 0:
+					time.sleep(storage['DISK_TIMEOUTS'])
+				else:
+					break
+
+			if cmd_handle.exit_code != 0:
+				raise DiskError(f'Could not encrypt volume "{partition.path}": {b"".join(cmd_handle)}')
 		except SysCallError as err:
 			if err.exit_code == 256:
 				log(f'{partition} is being used, trying to unmount and crypt-close the device and running one more attempt at encrypting the device.', level=logging.DEBUG)
@@ -91,7 +97,8 @@ class luks2():
 
 				# Get crypt-information about the device by doing a reverse lookup starting with the partition path
 				# For instance: /dev/sda
-				devinfo = json.loads(b''.join(sys_command(f"lsblk --fs -J {partition.path}")).decode('UTF-8'))['blockdevices'][0]
+				SysCommand(f'bash -c "partprobe"')
+				devinfo = json.loads(b''.join(SysCommand(f"lsblk --fs -J {partition.path}")).decode('UTF-8'))['blockdevices'][0]
 
 				# For each child (sub-partition/sub-device)
 				if len(children := devinfo.get('children', [])):
@@ -99,20 +106,17 @@ class luks2():
 						# Unmount the child location
 						if child_mountpoint := child.get('mountpoint', None):
 							log(f'Unmounting {child_mountpoint}', level=logging.DEBUG)
-							sys_command(f"umount -R {child_mountpoint}")
+							SysCommand(f"umount -R {child_mountpoint}")
 
 						# And close it if possible.
 						log(f"Closing crypt device {child['name']}", level=logging.DEBUG)
-						sys_command(f"cryptsetup close {child['name']}")
+						SysCommand(f"cryptsetup close {child['name']}")
 
 				# Then try again to set up the crypt-device
-				cmd_handle = sys_command(cryptsetup_args)
+				cmd_handle = SysCommand(cryptsetup_args)
 			else:
 				raise err
 
-		if cmd_handle.exit_code != 0:
-			raise DiskError(f'Could not encrypt volume "{partition.path}": {cmd_output}')
-	
 		return key_file
 
 	def unlock(self, partition, mountpoint, key_file):
@@ -124,6 +128,7 @@ class luks2():
 		:type mountpoint: str
 		"""
 		from .disk import get_filesystem_type
+
 		if '/' in mountpoint:
 			os.path.basename(mountpoint)  # TODO: Raise exception instead?
 
@@ -131,20 +136,40 @@ class luks2():
 		while pathlib.Path(partition.path).exists() is False and time.time() - wait_timer < 10:
 			time.sleep(0.025)
 
-		sys_command(f'/usr/bin/cryptsetup open {partition.path} {mountpoint} --key-file {os.path.abspath(key_file)} --type luks2')
+		SysCommand(f'/usr/bin/cryptsetup open {partition.path} {mountpoint} --key-file {os.path.abspath(key_file)} --type luks2')
 		if os.path.islink(f'/dev/mapper/{mountpoint}'):
 			self.mapdev = f'/dev/mapper/{mountpoint}'
 			unlocked_partition = Partition(self.mapdev, None, encrypted=True, filesystem=get_filesystem_type(self.mapdev), autodetect_filesystem=False)
-			unlocked_partition.allow_formatting = self.partition.allow_formatting
 			return unlocked_partition
 
 	def close(self, mountpoint=None):
 		if not mountpoint:
 			mountpoint = self.mapdev
 
-		sys_command(f'/usr/bin/cryptsetup close {self.mapdev}')
+		SysCommand(f'/usr/bin/cryptsetup close {self.mapdev}')
 		return os.path.islink(self.mapdev) is False
 
 	def format(self, path):
-		if (handle := sys_command(f"/usr/bin/cryptsetup -q -v luksErase {path}")).exit_code != 0:
+		if (handle := SysCommand(f"/usr/bin/cryptsetup -q -v luksErase {path}")).exit_code != 0:
 			raise DiskError(f'Could not format {path} with {self.filesystem} because: {b"".join(handle)}')
+
+	def add_key(self, path :pathlib.Path, password :str):
+		if not path.exists():
+			raise OSError(2, f"Could not import {path} as a disk encryption key, file is missing.", str(path))
+
+		log(f'Adding additional key-file {path} for {self.partition}', level=logging.INFO)
+
+		worker = SysCommandWorker(f"/usr/bin/cryptsetup -q -v luksAddKey {self.partition.path} {path}")
+		pw_injected = False
+		while worker.is_alive():
+			if b'Enter any existing passphrase' in worker and pw_injected is False:
+				worker.write(bytes(password, 'UTF-8'))
+				pw_injected = True
+
+		if worker.exit_code != 0:
+			raise DiskError(f'Could not add encryption key {path} to {self.partition} because: {worker}')
+
+	def crypttab(self, installation, key_path :str, options=["luks", "key-slot=1"]):
+		log(f'Adding a crypttab entry for key {key_path} in {installation}', level=logging.INFO)
+		with open(f"{installation.target}/etc/crypttab", "a") as crypttab:
+			crypttab.write(f"{self.mountpoint} UUID={convert_device_to_uuid(self.partition.path)} {key_path} {','.join(options)}\n")
